@@ -26,6 +26,9 @@
 import gtk
 from gtk import gdk, keysyms
 
+from kiwi.component import implements
+from kiwi.interfaces import IEasyCombo
+from kiwi.enums import ComboColumn, ComboMode
 from kiwi.ui.entry import KiwiEntry
 from kiwi.ui.entrycompletion import KiwiEntryCompletion
 from kiwi.utils import gsignal, type_register
@@ -309,27 +312,44 @@ class _ComboEntryPopup(gtk.Window):
     def get_selected_iter(self):
         model, treeiter = self._selection.get_selected()
 
-        # if the model currently being used is a TreeModelFiter, convert
+        # if the model currently being used is a TreeModelFilter, convert
         # the iter to be a TreeModel iter (witch is what the user expects)
         if isinstance(model, gtk.TreeModelFilter) and treeiter:
             treeiter = model.convert_iter_to_child_iter(treeiter)
         return treeiter
 
-    def set_selected_iter(self, iter):
+    def set_selected_iter(self, treeiter):
+        """
+        Selects an item in the comboentry given a treeiter
+        @param treeiter: the tree iter to select
+        """
         model = self._treeview.get_model()
 
         # Since the user passed a TreeModel iter, if the model currently
-        # being used is a TreeModelFiter, convert it to be a TreeModelFiter
+        # being used is a TreeModelFilter, convert it to be a TreeModelFilter
         # iter
         if isinstance(model, gtk.TreeModelFilter):
-            iter = model.convert_child_iter_to_iter(iter)
-        self._selection.select_iter(iter)
+            # See #3099 for an explanation why this is needed and a
+            # testcase
+            tmodel = model.get_model()
+            if tmodel.iter_is_valid(treeiter):
+               # revert back to the unfiltered model so we can select
+               # the right object
+               self._treeview.set_model(tmodel)
+               self._selection = self._treeview.get_selection()
+            else:
+                treeiter = model.convert_child_iter_to_iter(treeiter)
+        self._selection.select_iter(treeiter)
 
 type_register(_ComboEntryPopup)
 
 class ComboEntry(gtk.HBox):
+
+    implements(IEasyCombo)
+
     gsignal('changed')
     gsignal('activate')
+
     def __init__(self, entry=None):
         """
         @param entry: a gtk.Entry subclass to use
@@ -340,6 +360,7 @@ class ComboEntry(gtk.HBox):
         if not entry:
             entry = KiwiEntry()
 
+        self.mode = ComboMode.UNKNOWN
         self.entry = entry
         self.entry.connect('activate',
                            self._on_entry__activate)
@@ -473,8 +494,6 @@ class ComboEntry(gtk.HBox):
         """
         self._popup.popdown()
 
-    # Entry interface
-
     def set_text(self, text):
         """
         @param text:
@@ -487,8 +506,6 @@ class ComboEntry(gtk.HBox):
         """
         return self.entry.get_text()
 
-    # ComboMixin interface
-
     def set_model(self, model):
         """
         Set the tree model to model
@@ -526,9 +543,26 @@ class ComboEntry(gtk.HBox):
         """
         return self._popup.get_selected_iter()
 
+
+    def get_mode(self):
+        return self.mode
+
+    def set_label_text(self, text):
+        self._popup.set_label_text(text)
+
+    def set_active(self, rowno):
+        self.set_active_iter(self._model[rowno].iter)
+
+    # IEasyCombo interface
+
+    def clear(self):
+        """Removes all items from list"""
+        self._model.clear()
+        self.entry.set_text("")
+
     def prefill(self, itemdata, sort=False):
         """
-        See L{kiwi.ui.widgets.entry}
+        See L{kiwi.interfaces.IEasyCombo.prefill}
         """
         self._model.clear()
         self.entry.prefill(itemdata, sort)
@@ -536,22 +570,28 @@ class ComboEntry(gtk.HBox):
 
     def select_item_by_data(self, data):
         """
-        @param data: object to select
+        See L{kiwi.interfaces.IEasyCombo.select_item_by_data}
         """
         treeiter = self.entry.get_iter_by_data(data)
         self.set_active_iter(treeiter)
 
     def select_item_by_label(self, text):
         """
-        @param text: text to select
+        See L{kiwi.interfaces.IEasyCombo.select_item_by_label}
         """
         treeiter = self.entry.get_iter_by_label(text)
         self.set_active_iter(treeiter)
 
+    def select_item_by_position(self, position):
+        """
+        See L{kiwi.interfaces.IEasyCombo.select_item_by_position}
+        """
+        row = self._model[position]
+        self.set_active_iter(row.iter)
+
     def get_selected(self):
         """
-        @returns: selected text or item or None if nothing
-          is selected
+        See L{kiwi.interfaces.IEasyCombo.get_selected}
         """
         treeiter = self.get_active_iter()
         if treeiter:
@@ -559,21 +599,71 @@ class ComboEntry(gtk.HBox):
 
     def get_selected_label(self):
         """
-        @returns: the label of the currently selected item
+        See L{kiwi.interfaces.IEasyCombo.get_selected_label}
         """
         treeiter = self.get_active_iter()
         if treeiter:
             return self.entry.get_selected_label(treeiter)
 
+    def get_selected_data(self):
+        """
+        See L{kiwi.interfaces.IEasyCombo.get_selected_data}
+        """
+        treeiter = self.get_active_iter()
+        if treeiter:
+            return self.entry.get_selected_data(treeiter)
+
     def select(self, obj):
         """
-        @param obj: data or text to select
+        See L{kiwi.interfaces.IEasyCombo.select}
         """
         treeiter = self.entry.get_iter_from_obj(obj)
         self.set_active_iter(treeiter)
 
-    def set_label_text(self, text):
-        self._popup.set_label_text(text)
+    def append_item(self, label, data=None):
+        """
+        See L{kiwi.interfaces.IEasyCombo.append_item}
+        """
+        if not isinstance(label, basestring):
+            raise TypeError("label must be string, found %s" % label)
+
+        if self.mode == ComboMode.UNKNOWN:
+            if data is not None:
+                self.mode = ComboMode.DATA
+            else:
+                self.mode = ComboMode.STRING
+
+        model = self._model
+        if self.mode == ComboMode.STRING:
+            if data is not None:
+                raise TypeError("data can not be specified in string mode")
+            model.append((label, None))
+        elif self.mode == ComboMode.DATA:
+            if data is None:
+                raise TypeError("data must be specified in string mode")
+            model.append((label, data))
+        else:
+            raise AssertionError
+
+    def get_model_strings(self):
+        """
+        See L{kiwi.interfaces.IEasyCombo.get_model_strings}
+        """
+        return [row[ComboColumn.LABEL] for row in self._model]
+
+    def get_model_items(self):
+        """
+        See L{kiwi.interfaces.IEasyCombo.get_model_items}
+        """
+        if self.mode != ComboMode.DATA:
+            raise TypeError("get_model_items can only be used in data mode")
+
+        model = self._model
+        items = {}
+        for row in model:
+            items[row[ComboColumn.LABEL]] = row[ComboColumn.DATA]
+
+        return items
 
     # IconEntry
 
