@@ -36,8 +36,9 @@ from gtk import gdk
 from kiwi.accessor import kgetattr
 from kiwi.datatypes import converter, number, Decimal
 from kiwi.currency import currency # after datatypes
+from kiwi.enums import Alignment
 from kiwi.log import Logger
-from kiwi.python import slicerange
+from kiwi.python import enum, slicerange
 from kiwi.utils import PropertyObject, gsignal, gproperty, type_register
 
 _ = lambda m: gettext.dgettext('kiwi', m)
@@ -57,7 +58,71 @@ def str2bool(value, from_string=converter.from_string):
     return from_string(bool, value)
 
 class Column(PropertyObject, gobject.GObject):
-    """Specifies a column for an L{ObjectList}"""
+    """
+    Specifies a column for an L{ObjectList}, see the ObjectList documentation
+    for a simple example.
+
+    Properties
+    ==========
+      - B{title}: string I{mandatory}
+        - the title of the column, defaulting to the capitalized form of
+          the attribute
+      - B{data-type}: object I{str}
+        - the type of the attribute that will be inserted into the column.
+      - B{visible}: bool I{True}
+        - specifying if it is initially hidden or shown.
+      - B{justify}: gtk.Justification I{None}
+        - one of gtk.JUSTIFY_LEFT, gtk.JUSTIFY_RIGHT or gtk.JUSTIFY_CENTER
+          or None. If None, the justification will be determined by the type
+          of the attribute value of the first instance to be inserted in the
+          ObjectList (for instance numbers will be right-aligned).
+      - B{format}: string I{""}
+        - a format string to be applied to the attribute value upon insertion
+          in the list.
+      - B{width}: integer I{65535}
+        - the width in pixels of the column, if not set, uses the default to
+          ObjectList. If no Column specifies a width, columns_autosize() will
+          be called on the ObjectList upon append() or the first add_list().
+      - B{sorted}: bool I{False}
+        - whether or not the ObjectList is to be sorted by this column.
+          If no Columns are sorted, the ObjectList will be created unsorted.
+      - B{order}: GtkSortType I{-1}
+        - one of gtk.SORT_ASCENDING, gtk.SORT_DESCENDING or -1
+          The value -1 is mean that the column is not sorted.
+      - B{expand}: bool I{False}
+        - if set column will expand. Note: this space is shared equally amongst
+          all columns that have the expand set to True.
+      - B{tooltip}: string I{""}
+        - a string which will be used as a tooltip for the column header
+      - B{format_func}: object I{None}
+        -  a callable which will be used to format the output of a column.
+           The function will take one argument which is the value to convert
+           and is expected to return a string.
+           I{Note}: that you cannot use format and format_func at the same time,
+           if you provide a format function you'll be responsible for
+           converting the value to a string.
+      - B{editable}: bool I{False}
+        - if true the field is editable and when you modify the contents of
+          the cell the model will be updated.
+      - B{searchable}: bool I{False}
+        - if true the attribute values of the column can be searched using
+          type ahead search. Only string attributes are currently supported.
+      - B{radio}: bool I{False}
+        -  If true render the column as a radio instead of toggle.
+           Only applicable for columns with boolean data types.
+      - B{cache}: bool I{False}
+        -  If true, the value will only be fetched once, and the same value
+           will be reused for futher access.
+      - B{use_stock}: bool I{False}
+        - If true, this will be rendered as pixbuf from the value which
+          should be a stock id.
+      - B{icon_size}: gtk.IconSize I{gtk.ICON_SIZE_MENU}
+      - B{editable_attribute}: string I{""}
+        - a string which is the attribute which should decide if the
+          cell is editable or not.
+      - B{use_markup}: bool I{False}
+        - If true, the text will be rendered with markup
+    """
     gproperty('title', str)
     gproperty('data-type', object)
     gproperty('visible', bool, default=True)
@@ -103,50 +168,6 @@ class Column(PropertyObject, gobject.GObject):
         @param data_type: the type of the attribute that will be inserted
             into the column.
 
-        @keyword visible: a boolean specifying if it is initially hidden or
-            shown.
-        @keyword justify: one of gtk.JUSTIFY_LEFT, gtk.JUSTIFY_RIGHT or
-            gtk.JUSTIFY_CENTER or None. If None, the justification will be
-            determined by the type of the attribute value of the first
-            instance to be inserted in the ObjectList (numbers will be
-            right-aligned).
-        @keyword format: a format string to be applied to the attribute
-            value upon insertion in the list.
-        @keyword width: the width in pixels of the column, if not set, uses the
-            default to ObjectList. If no Column specifies a width,
-            columns_autosize() will be called on the ObjectList upon append()
-            or the first add_list().
-        @keyword sorted: whether or not the ObjectList is to be sorted by this
-            column.
-            If no Columns are sorted, the ObjectList will be created unsorted.
-        @keyword order: one of gtk.SORT_ASCENDING or gtk.SORT_DESCENDING or
-            -1. The value -1 is used internally when the column is not sorted.
-        @keyword expand: if set column will expand. Note: this space is shared
-            equally amongst all columns that have the expand set to True.
-        @keyword tooltip: a string which will be used as a tooltip for
-            the column header
-        @keyword format_func: a callable which will be used to format
-            the output of a column. The function will take one argument
-            which is the value to convert and is expected to return a string.
-            Note that you cannot use format and format_func at the same time,
-            if you provide a format function you'll be responsible for
-            converting the value to a string.
-        @keyword editable: if true the field is editable and when you modify
-            the contents of the cell the model will be updated.
-        @keyword searchable: if true the attribute values of the column can
-            be searched using type ahead search. Only string attributes are
-            currently supported.
-        @keyword radio: If true render the column as a radio instead of toggle.
-            Only applicable for columns with boolean data types.
-        @keyword cache: If true, the value will only be fetched once,
-            and the same value will be reused for futher access.
-        @keyword use_stock: If true, this will be rendered as pixbuf from the
-            value which should be a stock id.
-        @keyword icon_size: a gtk.IconSize constant, gtk.ICON_SIZE_MENU if not
-            specified.
-        @keyword editable_attribute: a string which is the attribute
-            which should decide if the cell is editable or not.
-        @keyword use_markup: If true, the text will be rendered with markup
         @keyword title_pixmap: (TODO) if set to a filename a pixmap will be
             used *instead* of the title set. The title string will still be
             used to identify the column in the column selection and in a
@@ -172,9 +193,10 @@ class Column(PropertyObject, gobject.GObject):
         # center for bools and left align it for everything else.
         if "justify" not in kwargs:
             if data_type:
+                conv = converter.get_converter(data_type)
                 if issubclass(data_type, bool):
                     kwargs['justify'] = gtk.JUSTIFY_CENTER
-                elif issubclass(data_type, (number, currency)):
+                elif conv.align == Alignment.RIGHT:
                     kwargs['justify'] = gtk.JUSTIFY_RIGHT
 
         format_func = kwargs.get('format_func')
@@ -220,7 +242,8 @@ class Column(PropertyObject, gobject.GObject):
             data_type == currency or
             data_type == datetime.date or
             data_type == datetime.datetime or
-            data_type == datetime.time):
+            data_type == datetime.time or
+            issubclass(data_type, enum)):
             conv = converter.get_converter(data_type)
             text = conv.as_string(data, format=self.format or None)
         elif self.format_func:
@@ -411,10 +434,35 @@ class ObjectList(PropertyObject, gtk.ScrolledWindow):
     >>> banana.name = 'Banana'
     >>> banana.description = 'Monkey food'
 
-    >>> list = ObjectList([Column('name'),
-    >>>                    Column('description')])
-    >>> list.append(apple)
-    >>> list.append(banana)
+    >>> fruits = ObjectList([Column('name'),
+    >>>                      Column('description')])
+    >>> fruits.append(apple)
+    >>> fruits.append(banana)
+
+    Signals
+    =======
+      - B{row-activated} (object):
+        - Emitted when a row is "activated", eg double clicked or pressing
+          enter. See the GtkTreeView documentation for more information
+      - B{selection-changed} (object):
+        - Emitted when the selection changes for the ObjectList
+          enter. See the documentation on GtkTreeSelection::changed
+          for more information
+      - B{double-click} (object):
+        - Emitted when a row is double-clicked, mostly you want to use
+          the row-activated signal instead to be able catch keyboard events.
+      - B{right-click} (object):
+        - Emitted when a row is clicked with the right mouse button.
+      - B{cell-edited} (object, attribute):
+        - Emitted when a cell is edited.
+      - B{has-rows} (bool):
+        - Emitted when the objectlist goes from an empty to a non-empty
+          state or vice verse.
+
+    Properties
+    ==========
+      - B{selection-mode}: gtk.SelectionMode I{gtk.SELECTION_BROWSE}
+        - Represents the selection-mode of a GtkTreeSelection of a GtkTreeView.
     """
 
     __gtype_name__ = 'ObjectList'
@@ -428,6 +476,9 @@ class ObjectList(PropertyObject, gtk.ScrolledWindow):
     # row double-clicked
     gsignal('double-click', object)
 
+    # row right-clicked
+    gsignal('right-click', object)
+
     # edited object, attribute name
     gsignal('cell-edited', object, str)
 
@@ -788,6 +839,8 @@ class ObjectList(PropertyObject, gtk.ScrolledWindow):
 
         if column.use_stock:
             cell_data_func = self._cell_data_pixbuf_func
+        elif issubclass(column.data_type, enum):
+            cell_data_func = self._cell_data_combo_func
         else:
             cell_data_func = self._cell_data_text_func
 
@@ -884,6 +937,25 @@ class ObjectList(PropertyObject, gtk.ScrolledWindow):
             prop = 'pixbuf'
             if column.editable:
                 raise TypeError("use-stock columns cannot be editable")
+        elif issubclass(data_type, enum):
+            if data_type is enum:
+                raise TypeError("data_type must be a subclass of enum")
+
+            model = gtk.ListStore(str, object)
+            items = data_type.names.items()
+            items.sort()
+            for key, value in items:
+                model.append((key.lower().capitalize(), value))
+
+            renderer =  gtk.CellRendererCombo()
+            renderer.set_property('model', model)
+            renderer.set_property('text-column', 0)
+            renderer.set_property('has-entry', True)
+            if column.editable:
+                renderer.set_property('editable', True)
+                renderer.connect('edited', self._on_renderer_combo__edited,
+                                 self._model, column.attribute, column)
+            prop = 'model'
         elif issubclass(data_type, (datetime.date, datetime.time,
                                     basestring, number,
                                     currency)):
@@ -987,15 +1059,25 @@ class ObjectList(PropertyObject, gtk.ScrolledWindow):
 
     def _on_treeview__button_press_event(self, treeview, event):
         "Generic button-press-event handler to be able to catch double clicks"
-        if event.type == gtk.gdk._2BUTTON_PRESS:
-            selection = self._treeview.get_selection()
-            mode = selection.get_mode()
-            if mode == gtk.SELECTION_MULTIPLE:
-                item = self.get_selected_rows()
-            else:
-                item = self.get_selected()
+
+        selection = self._treeview.get_selection()
+        mode = selection.get_mode()
+        if mode == gtk.SELECTION_MULTIPLE:
+            item = self.get_selected_rows()
+        else:
+            item = self.get_selected()
+
+        if not item:
+            return
+
+        # Double left click
+        if event.type == gtk.gdk._2BUTTON_PRESS and event.button == 1:
             self.emit('double-click', item)
 
+        # Right click
+        if event.type == gtk.gdk.BUTTON_PRESS and event.button == 3:
+            self.emit('right-click', item)
+
     # CellRenderers
     def _cell_data_text_func(self, tree_column, renderer, model, treeiter,
                              (column, renderer_prop)):
@@ -1041,6 +1123,14 @@ class ObjectList(PropertyObject, gtk.ScrolledWindow):
         pixbuf = self.render_icon(data, column.icon_size)
         renderer.set_property(renderer_prop, pixbuf)
 
+    def _cell_data_combo_func(self, tree_column, renderer, model, treeiter,
+                              (column, renderer_prop)):
+        row = model[treeiter]
+        data = column.get_attribute(row[COL_MODEL],
+                                    column.attribute, None)
+        text = column.as_string(data)
+        renderer.set_property('text', text.lower().capitalize())
+
     def _on_renderer__toggled(self, renderer, path, column):
         setattr(self._model[path][COL_MODEL], column.attribute,
                 not renderer.get_active())
@@ -1087,6 +1177,22 @@ class ObjectList(PropertyObject, gtk.ScrolledWindow):
         setattr(obj, attr, value)
         self.emit('cell-edited', obj, attr)
 
+    def _on_renderer_combo__edited(self, renderer, path, text,
+                                  model, attr, column):
+        obj = model[path][COL_MODEL]
+        if not text:
+            return
+
+        value_model = renderer.get_property('model')
+        for row in value_model:
+            if row[0] == text:
+                value = row[1]
+                break
+        else:
+            raise AssertionError
+        setattr(obj, attr, value)
+        self.emit('cell-edited', obj, attr)
+
     def _on_renderer__edited(self, renderer, path, value, column):
         data_type = column.data_type
         if data_type in number:
@@ -1491,11 +1597,28 @@ class ObjectList(PropertyObject, gtk.ScrolledWindow):
         """
         self._treeview.set_headers_visible(value)
 
-type_register(ObjectList)
+    def set_visible_rows(self, rows):
+        """
+        Sets the number of visible rows of the treeview. This is useful to use
+        instead of set_size_request() directly, since you can avoid using raw
+        pixel sizes.
+        @param rows: number of rows to show
+        """
 
-class ObjectRow(object):
-    def __init__(self, iter):
-        self.iter = iter
+        treeview = self._treeview
+        if treeview.get_headers_visible():
+            treeview.realize()
+            header_h = self._get_header_height()
+        else:
+            header_h = 0
+
+        column = treeview.get_columns()[0]
+        h = column.cell_get_size()[-1]
+
+        focus_padding = treeview.style_get_property('focus-line-width') * 2
+        treeview.set_size_request(-1, header_h + (rows * (h + focus_padding)))
+
+type_register(ObjectList)
 
 class ObjectTree(ObjectList):
     def __init__(self, columns=[], objects=None, mode=gtk.SELECTION_BROWSE,
@@ -1505,17 +1628,18 @@ class ObjectTree(ObjectList):
         ObjectList.__init__(self, columns, objects, mode, sortable, model)
 
     def _append_internal(self, parent, instance, select, prepend):
-        if parent is not None and not isinstance(parent, ObjectRow):
-            raise TypeError("parent must be an ObjectRow or None")
+        iters = self._iters
+        parent_id = id(parent)
+        if parent_id in iters:
+            parent_iter = iters[parent_id]
+        elif parent is None:
+            parent_iter = None
+        else:
+            raise TypeError("parent must be an Object, ObjectRow or None")
 
         # Freeze and save original selection mode to avoid blinking
         self._treeview.freeze_notify()
 
-        if parent:
-            parent_iter = parent.iter
-        else:
-            parent_iter = None
-
         if prepend:
             row_iter = self._model.prepend(parent_iter, (instance,))
         else:
@@ -1530,24 +1654,56 @@ class ObjectTree(ObjectList):
             self._select_and_focus_row(row_iter)
         self._treeview.thaw_notify()
 
-        return ObjectRow(row_iter)
+        return instance
 
     def append(self, parent, instance, select=False):
         """
-        @param parent: ObjectRow of the parent
+        @param parent: Object or None, representing the parent
         @param instance: the instance to be added
         @param select: select the row
+        @returns: the appended object
         """
         return self._append_internal(parent, instance, select, prepend=False)
 
     def prepend(self, parent, instance, select=False):
         """
-        @param parent: ObjectRow of the parent
+        @param parent: Object or None, representing the parent
         @param instance: the instance to be added
         @param select: select the row
+        @returns: the prepended object
         """
         return self._append_internal(parent, instance, select, prepend=True)
 
+    def expand(self, instance, open_all=True):
+        """
+        This method opens the row specified by path so its children
+        are visible.
+        @param instance: an instance to expand at
+        @param open_all: If True, expand all rows, otherwise just the
+        immediate children
+        """
+        objid = id(instance)
+        if not objid in self._iters:
+            raise ValueError("instance %r is not in the list" % instance)
+        treeiter = self._iters[objid]
+        
+        self.get_treeview().expand_row(
+            self._model[treeiter].path, open_all)
+        
+    def collapse(self, instance):
+        """
+        This method collapses the row specified by path
+        (hides its child rows, if they exist).
+        @param instance: an instance to collapse
+        """
+        objid = id(instance)
+        if not objid in self._iters:
+            raise ValueError("instance %r is not in the list" % instance)
+        treeiter = self._iters[objid]
+
+        self.get_treeview().collapse_row(
+            self._model[treeiter].path)
+
 type_register(ObjectTree)
 
 class ListLabel(gtk.HBox):
