@@ -1,25 +1,59 @@
-from decimal import Decimal
+import cPickle
 import datetime
-import unittest
 import locale
+import unittest
+import sys
 
-from kiwi.datatypes import currency, converter, ValidationError, ValueUnset
+from gtk import gdk
 
-import utils
+from kiwi.datatypes import converter, ValidationError, ValueUnset, \
+     Decimal, BaseConverter
+from kiwi.currency import currency
+from kiwi.environ import environ
+from kiwi.python import enum
+
+# pixbuf converter
+from kiwi.ui import proxywidget
+proxywidget # pyflakes
 
 def set_locale(category, name):
     # set the date format to the spanish one
     try:
-        locale.setlocale(category, name)
+        rv = locale.setlocale(category, name)
     except locale.Error:
         print 'skipping %s, locale not available' % name
         return False
     return True
 
+fake = type('fake', (object,), {})
+class FakeConverter(BaseConverter):
+    type = fake
+
 class RegistryTest(unittest.TestCase):
     def testAdd(self):
         self.assertRaises(TypeError, converter.add, object)
 
+    def testRemove(self):
+        self.assertRaises(TypeError, converter.remove, object)
+
+    def testFake(self):
+        self.assertRaises(KeyError, converter.remove, FakeConverter)
+        converter.add(FakeConverter)
+        self.assertRaises(ValueError, converter.add, FakeConverter)
+        converter.remove(FakeConverter)
+        self.assertRaises(KeyError, converter.remove, FakeConverter)
+
+    def testGetConverters(self):
+        converters = converter.get_converters((Decimal,))
+        # Curreny is a subclass of Decimal, so it should be in converters
+        conv = converter.get_converter(currency)
+        self.assertTrue(conv in converters)
+
+        converters = converter.get_converters((object,))
+        # Object is treated specially. Make sure its in the list
+        conv = converter.get_converter(object)
+        self.assertTrue(conv in converters)
+        
 class BoolTest(unittest.TestCase):
     def setUp(self):
         self.conv = converter.get_converter(bool)
@@ -39,11 +73,12 @@ class BoolTest(unittest.TestCase):
 
 class DateTest(unittest.TestCase):
     def setUp(self):
+        set_locale(locale.LC_TIME, 'C')
         self.date = datetime.date(1979, 2, 12)
         self.conv = converter.get_converter(datetime.date)
 
     def tearDown(self):
-        set_locale(locale.LC_ALL, 'C')
+        set_locale(locale.LC_TIME, 'C')
 
     def testFromStringES(self):
         if not set_locale(locale.LC_TIME, 'es_ES'):
@@ -77,21 +112,46 @@ class DateTest(unittest.TestCase):
 
         self.assertEqual(self.conv.as_string(self.date), "12-02-1979")
 
+    def testFromStringPortugueseBrazil(self):
+        if not set_locale(locale.LC_TIME, 'Portuguese_Brazil.1252'):
+            return
+
+        self.assertEqual(self.conv.from_string("12/2/1979"), self.date)
+        self.assertEqual(self.conv.from_string("12/02/1979"), self.date)
+
+        # test some invalid dates
+        self.assertRaises(ValidationError,
+                          self.conv.from_string, "40/10/2005")
+        self.assertRaises(ValidationError,
+                          self.conv.from_string, "30/02/2005")
+
+    def testAsStringPortugueseBrazil(self):
+        if not set_locale(locale.LC_TIME, 'Portuguese_Brazil.1252'):
+            return
+
+        self.assertEqual(self.conv.as_string(self.date), "12/02/1979")
+
 class CurrencyTest(unittest.TestCase):
     def setUp(self):
+        set_locale(locale.LC_ALL, 'C')
         self.conv = converter.get_converter(currency)
 
     def tearDown(self):
         set_locale(locale.LC_ALL, 'C')
 
     def testFormatBR(self):
-        if not set_locale(locale.LC_MONETARY, 'pt_BR'):
+        if not set_locale(locale.LC_ALL, 'pt_BR'):
             return
 
-        self.assertEqual(currency(100).format(), 'R$100')
-        self.assertEqual(currency('123.45').format(), 'R$123,45')
-        self.assertEqual(currency(12345).format(), 'R$12.345')
-        self.assertEqual(currency(-100).format(), 'R$-100')
+        self.assertEqual(currency(100).format(), 'R$ 100')
+        self.assertEqual(currency('123,45').format(), 'R$ 123,45')
+        self.assertEqual(currency(12345).format(), 'R$ 12.345')
+        self.assertEqual(currency(-100).format(), 'R$ -100')
+        try:
+            c = currency('R$1.234.567,40')
+        except:
+            raise AssertionError("monetary separator could not be removed")
+        self.assertEqual(c, Decimal('1234567.40'))
 
         # Sometimes it works, sometimes it doesn''10,000,000.0't
         #self.assertEqual(self.conv.from_string('0,5'), currency('0.5'))
@@ -117,6 +177,27 @@ class CurrencyTest(unittest.TestCase):
         self.assertEqual(self.conv.as_string(currency(-10)), '$-10.00')
         #self.assertEqual(ValidationError, self.conv.as_string, object)
 
+    def testPickle(self):
+        pickled_var = cPickle.dumps(currency("123.45"))
+        recoverd_var = cPickle.loads(pickled_var)
+        self.assertEqual(recoverd_var.format(), '$123.45')
+
+    def testPickleBR(self):
+        if not set_locale(locale.LC_ALL, 'pt_BR'):
+            return
+
+        pickled_var = cPickle.dumps(currency("123.45"))
+        recoverd_var = cPickle.loads(pickled_var)
+        self.assertEqual(recoverd_var.format(), 'R$ 123,45')
+
+    def testPickleUS(self):
+        if not set_locale(locale.LC_ALL, 'en_US'):
+            return
+
+        pickled_var = cPickle.dumps(currency("12123.45"))
+        recoverd_var = cPickle.loads(pickled_var)
+        self.assertEqual(recoverd_var.format(), '$12,123.45')
+
 class UnicodeTest(unittest.TestCase):
     def setUp(self):
         self.conv = converter.get_converter(unicode)
@@ -227,10 +308,11 @@ class FloatTest(unittest.TestCase):
 
 class DecimalTest(unittest.TestCase):
     def setUp(self):
+        set_locale(locale.LC_NUMERIC, 'C')
         self.conv = converter.get_converter(Decimal)
 
     def tearDown(self):
-        set_locale(locale.LC_ALL, 'C')
+        set_locale(locale.LC_NUMERIC, 'C')
 
     def testFromString(self):
         self.assertEqual(self.conv.from_string('-2.5'), Decimal('-2.5'))
@@ -243,16 +325,22 @@ class DecimalTest(unittest.TestCase):
         self.assertEqual(self.conv.as_string(Decimal('0.0')), '0.0')
         self.assertEqual(self.conv.as_string(Decimal('0.5')), '0.5')
         self.assertEqual(self.conv.as_string(Decimal('-0.5')), '-0.5')
-        self.assertEqual(self.conv.as_string(Decimal('0.123456789')), '0.123456789')
-        self.assertEqual(self.conv.as_string(Decimal('-0.123456789')), '-0.123456789')
-        self.assertEqual(self.conv.as_string(Decimal('10000000')), '10000000.0')
-        self.assertEqual(self.conv.as_string(Decimal('10000000.0')), '10000000.0')
+        self.assertEqual(self.conv.as_string(Decimal('0.123456789')),
+                         '0.123456789')
+        self.assertEqual(self.conv.as_string(Decimal('-0.123456789')),
+                         '-0.123456789')
+        self.assertEqual(self.conv.as_string(Decimal('10000000')),
+                         '10000000.0')
+        self.assertEqual(self.conv.as_string(Decimal('10000000.0')),
+                         '10000000.0')
 
     def testAsStringUS(self):
         if not set_locale(locale.LC_NUMERIC, 'en_US'):
             return
-        self.assertEqual(self.conv.as_string(Decimal('10000000')), '10,000,000.0')
-        self.assertEqual(self.conv.as_string(Decimal('10000000.0')), '10,000,000.0')
+        self.assertEqual(self.conv.as_string(Decimal('10000000')),
+                         '10,000,000.0')
+        self.assertEqual(self.conv.as_string(Decimal('10000000.0')),
+                         '10,000,000.0')
 
     def testAsStringSE(self):
         if not set_locale(locale.LC_NUMERIC, 'sv_SE'):
@@ -260,12 +348,62 @@ class DecimalTest(unittest.TestCase):
         self.assertEqual(self.conv.as_string(Decimal('0.0')), '0,0')
         self.assertEqual(self.conv.as_string(Decimal('0.5')), '0,5')
         self.assertEqual(self.conv.as_string(Decimal('-0.5')), '-0,5')
-        self.assertEqual(self.conv.as_string(Decimal('0.123456789')), '0,123456789')
-        self.assertEqual(self.conv.as_string(Decimal('-0.123456789')), '-0,123456789')
-        self.assertEqual(self.conv.as_string(Decimal('10000000')), '10000000,0')
-        self.assertEqual(self.conv.as_string(Decimal('10000000.0')), '10000000,0')
+        self.assertEqual(self.conv.as_string(Decimal('0.123456789')),
+                         '0,123456789')
+        self.assertEqual(self.conv.as_string(Decimal('-0.123456789')),
+                         '-0,123456789')
+        self.assertEqual(self.conv.as_string(Decimal('10000000')),
+                         '10000000,0')
+        self.assertEqual(self.conv.as_string(Decimal('10000000.0')),
+                         '10000000,0')
 
+class PixbufTest(unittest.TestCase):
+    def setUp(self):
+        self.conv = converter.get_converter(gdk.Pixbuf)
 
+    def testPNGAsString(self):
+        if sys.platform == 'win32':
+            return
+        file_name = environ.find_resource('pixmap', 'validation-error-16.png')
+
+        f = file(file_name)
+        png_string = f.read()
+        f.close()
+
+        pixbuf = self.conv.from_string(png_string)
+        string = self.conv.as_string(pixbuf)
+        # XXX Not always equal. need to investigate
+        #self.assertEqual(string, png_string)
+
+        # Compare png header
+        self.assertEqual(string[0:8], '\x89\x50\x4e\x47\x0d\x0a\x1a\x0a')
+
+    def testPNGFromString(self):
+        if sys.platform == 'win32':
+            return
+        file_name = environ.find_resource('pixmap', 'validation-error-16.png')
+        f = file(file_name)
+        png_string = f.read()
+        f.close()
+
+        pixbuf = self.conv.from_string(png_string)
+        self.assertEqual(pixbuf.get_width(), 17)
+        self.assertEqual(pixbuf.get_height(), 17)
+
+class EnumTest(unittest.TestCase):
+    def testSimple(self):
+        class status(enum):
+            (OPEN, CLOSE) = range(2)
+
+        conv = converter.get_converter(status)
+        self.assertEqual(conv.type, status)
+        conv2 = converter.get_converter(status)
+        self.assertEqual(conv, conv2)
+
+        self.assertEqual(conv.from_string('OPEN'), status.OPEN)
+        self.assertEqual(conv.as_string(status.CLOSE), 'CLOSE')
+        self.assertRaises(ValidationError, conv.from_string, 'FOO')
+        self.assertRaises(ValidationError, conv.as_string, object())
 
 if __name__ == "__main__":
     unittest.main()
